#include <stdio.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/time.h>

float *calculate_on_host(const float *a, const float *b, size_t n) {
	float *c = malloc(sizeof(*c) * n*n);
    float dotproduct;
    for(size_t i = 0; i < n; i++) {
        for(size_t j = 0; j < n; j++) {
            dotproduct = 0.0;
            for(size_t k = 0; k < n; k++) {
                dotproduct += a[i*n+k] * b[k*n+j];
            }
            c[i*n+j] = dotproduct;
        }
    }
    return c;
}

float *generate_input(size_t n) {

    float *data = malloc(sizeof(*data) * n * n);
    for (int i = 0; i < n * n; i++)
        // data[i] = i;
        data[i] = drand48() - 0.5;

    return data;
}


double walltime (void) {
	static struct timeval t;
	gettimeofday (&t, NULL);
	return t.tv_sec + 1e-6 * t.tv_usec;
}

void print_matrix(float *a, size_t n) {
    for(size_t i = 0; i < n; i++) {
        for(size_t j = 0; j < n; j++) {
            printf("%8.2f ", a[i*n+j]);
        }
        printf("\n");
    }
    printf("\n");
}

int verify(const float *a1, const float *a2, int n) {
	for (int i = 0; i < n; i++) {
        for (int j = 0; j < n; j++) {
            int index = i * n + j;
    		if (fabs(a1[index]-a2[index]) > 0.001) {
                printf("host[%d][%d] (%.3f) != device[%d][%d] (%.3f)\n", i, j, a1[index], i, j, a2[index]);
    			return 0;
    		}
        }
    }
	return 1;
}