#include <string.h>

#ifndef CL_DEVICE_TYPE_GPU
#define CL_DEVICE_TYPE_GPU 0
#define CL_DEVICE_TYPE_CPU 1

typedef int cl_device_type;

#endif

struct args_t
{
    int verbose;
    cl_device_type device;
    size_t matrix_size;
    size_t block_size;
    int use_files;
};

struct args_t parse_args(int argc, char const **argv) {
    struct args_t args;
    args.verbose = 0;
    args.device = CL_DEVICE_TYPE_CPU;
    args.matrix_size = 128;
    args.block_size = 4;
    args.use_files = 1;
    
    for (int i = 1; i < argc; i++) {
        if (!strcmp(argv[i], "-v"))
            args.verbose = 1;
        else if (!strcmp(argv[i], "-vv"))
            args.verbose = 2;
        else if (!strcmp(argv[i], "-nofiles"))
            args.use_files = 0;
        else if (!strcmp(argv[i], "-b")) {
            args.block_size = atol(argv[i+1]);
            i++;
        } else if (!strcmp(argv[i], "-gpu"))
            args.device = CL_DEVICE_TYPE_GPU;
        else if (!strcmp(argv[i], "-cpu"))
            args.device = CL_DEVICE_TYPE_CPU;
        else {
            int base = atol(argv[i]);
            if (i + 1 < argc) {
                int ex = atol(argv[i+1]);
                base = pow(base, ex);
            }
            args.matrix_size = base;
            return args;
        }
    }
    return args;
}

