void run(int argc, const char **argv) {
    float *a, *b, *hc;
    srand48(time(NULL));
    
    struct args_t args = parse_args(argc, argv);
    size_t n = args.matrix_size;
    
    int use_files = args.use_files && files_exist(n);
    
    if (args.verbose > 0) {
        printf("Args:\n");
        printf("  %lu x %lu matrix\n", n, n);
        printf("  device: %s \n", args.device == CL_DEVICE_TYPE_GPU ? "GPU" : "CPU");
        printf("  verbose: %i \n", args.verbose);
        printf("  files: %s \n", use_files ? "yes" : "no");
    }

    if (use_files) {
        load_matrix_files(&a, &b, &hc, n);
    } else {
        a = generate_input(n);
        b = generate_input(n);
    }
    
    float *ha = malloc(sizeof(*a) * n * n);
    float *hb = malloc(sizeof(*a) * n * n);
    float *da = malloc(sizeof(*a) * n * n);
    float *db = malloc(sizeof(*a) * n * n);

    memcpy(ha, a, sizeof(*a) * n * n);
    memcpy(hb, b, sizeof(*a) * n * n);
    memcpy(da, a, sizeof(*a) * n * n);
    memcpy(db, b, sizeof(*a) * n * n);

    if (!use_files) {
        hc = calculate_on_host(ha, hb, n);
    }
    
    start_listening();
    float *dc = calculate_on_device(da, db, n, args);
    stop_listening();
    
    if (args.verbose == 2) {
        printf("Original\n");
        print_matrix(a, n);
        print_matrix(b, n);
    
        printf("Host:\n");
        print_matrix(hc, n);
        printf("Device:\n");
        print_matrix(dc, n);
    }
    
    int check = 1;
    for(size_t i = 0; i < n; i++) {
        for(size_t j = 0; j < n; j++) {
            if (fabs(hc[i*n+j] - dc[i*n+j]) > 0.001) {
                printf("host[%lu][%lu] (%.3f) != device[%lu][%lu] (%.3f)\n", i, j, hc[i*n+j], i, j, dc[i*n+j]);
                check = 0;
                break;
            }
        }
        if (!check) break;
    }
    
    if (args.verbose == 0) {
        printf("%d ", check);
        print_counters(args.verbose);
        printf("\n");
    } else {
        printf("Verify  : %d\n", check);
        print_counters(args.verbose);
    }
    if (check && !use_files) {
        write_matrix_files(a, b, hc, n);
    }
    
    free(a);
    free(b);
    
    free(ha);
    free(hb);
    free(hc);
    
    free(da);
    free(db);
    free(dc);
}
