__kernel __attribute__((vec_type_hint(float8)))
void transpose(__global float8 *g_mat) {
    uint size = NUMBLOCKS;
    __global float8 *src, *dst;
    float8 l_mat[8];
    
    uint col = get_global_id(0);
    uint row = 0;
    while (col >= size) {
        col -= size--;
        row++;
    }
    col += row;
    size += row;
    
    src = g_mat + row * size * 8 + col;
    l_mat[0] = src[0];
    l_mat[1] = src[size];
    l_mat[2] = src[2*size];
    l_mat[3] = src[3*size];
    l_mat[4] = src[4*size];
    l_mat[5] = src[5*size];
    l_mat[6] = src[6*size];
    l_mat[7] = src[7*size];
    
    if (row == col) {
        src[0]      = (float8) (l_mat[0].s0, l_mat[1].s0, l_mat[2].s0, l_mat[3].s0, l_mat[4].s0, l_mat[5].s0, l_mat[6].s0, l_mat[7].s0);
        src[size]   = (float8) (l_mat[0].s1, l_mat[1].s1, l_mat[2].s1, l_mat[3].s1, l_mat[4].s1, l_mat[5].s1, l_mat[6].s1, l_mat[7].s1);
        src[2*size] = (float8) (l_mat[0].s2, l_mat[1].s2, l_mat[2].s2, l_mat[3].s2, l_mat[4].s2, l_mat[5].s2, l_mat[6].s2, l_mat[7].s2);
        src[3*size] = (float8) (l_mat[0].s3, l_mat[1].s3, l_mat[2].s3, l_mat[3].s3, l_mat[4].s3, l_mat[5].s3, l_mat[6].s3, l_mat[7].s3);
        src[4*size] = (float8) (l_mat[0].s4, l_mat[1].s4, l_mat[2].s4, l_mat[3].s4, l_mat[4].s4, l_mat[5].s4, l_mat[6].s4, l_mat[7].s4);
        src[5*size] = (float8) (l_mat[0].s5, l_mat[1].s5, l_mat[2].s5, l_mat[3].s5, l_mat[4].s5, l_mat[5].s5, l_mat[6].s5, l_mat[7].s5);
        src[6*size] = (float8) (l_mat[0].s6, l_mat[1].s6, l_mat[2].s6, l_mat[3].s6, l_mat[4].s6, l_mat[5].s6, l_mat[6].s6, l_mat[7].s6);
        src[7*size] = (float8) (l_mat[0].s7, l_mat[1].s7, l_mat[2].s7, l_mat[3].s7, l_mat[4].s7, l_mat[5].s7, l_mat[6].s7, l_mat[7].s7);
    } else {
        dst = g_mat + col * size * 8 + row;
        
        src[0]      = (float8) (dst[0].s0, dst[size].s0, dst[2*size].s0, dst[3*size].s0, dst[4*size].s0, dst[5*size].s0, dst[6*size].s0, dst[7*size].s0);
        src[size]   = (float8) (dst[0].s1, dst[size].s1, dst[2*size].s1, dst[3*size].s1, dst[4*size].s1, dst[5*size].s1, dst[6*size].s1, dst[7*size].s1);
        src[2*size] = (float8) (dst[0].s2, dst[size].s2, dst[2*size].s2, dst[3*size].s2, dst[4*size].s2, dst[5*size].s2, dst[6*size].s2, dst[7*size].s2);
        src[3*size] = (float8) (dst[0].s3, dst[size].s3, dst[2*size].s3, dst[3*size].s3, dst[4*size].s3, dst[5*size].s3, dst[6*size].s3, dst[7*size].s3);
        src[4*size] = (float8) (dst[0].s4, dst[size].s4, dst[2*size].s4, dst[3*size].s4, dst[4*size].s4, dst[5*size].s4, dst[6*size].s4, dst[7*size].s4);
        src[5*size] = (float8) (dst[0].s5, dst[size].s5, dst[2*size].s5, dst[3*size].s5, dst[4*size].s5, dst[5*size].s5, dst[6*size].s5, dst[7*size].s5);
        src[6*size] = (float8) (dst[0].s6, dst[size].s6, dst[2*size].s6, dst[3*size].s6, dst[4*size].s6, dst[5*size].s6, dst[6*size].s6, dst[7*size].s6);
        src[7*size] = (float8) (dst[0].s7, dst[size].s7, dst[2*size].s7, dst[3*size].s7, dst[4*size].s7, dst[5*size].s7, dst[6*size].s7, dst[7*size].s7);

        dst[0]      = (float8) (l_mat[0].s0, l_mat[1].s0, l_mat[2].s0, l_mat[3].s0, l_mat[4].s0, l_mat[5].s0, l_mat[6].s0, l_mat[7].s0);
        dst[size]   = (float8) (l_mat[0].s1, l_mat[1].s1, l_mat[2].s1, l_mat[3].s1, l_mat[4].s1, l_mat[5].s1, l_mat[6].s1, l_mat[7].s1);
        dst[2*size] = (float8) (l_mat[0].s2, l_mat[1].s2, l_mat[2].s2, l_mat[3].s2, l_mat[4].s2, l_mat[5].s2, l_mat[6].s2, l_mat[7].s2);
        dst[3*size] = (float8) (l_mat[0].s3, l_mat[1].s3, l_mat[2].s3, l_mat[3].s3, l_mat[4].s3, l_mat[5].s3, l_mat[6].s3, l_mat[7].s3);
        dst[4*size] = (float8) (l_mat[0].s4, l_mat[1].s4, l_mat[2].s4, l_mat[3].s4, l_mat[4].s4, l_mat[5].s4, l_mat[6].s4, l_mat[7].s4);
        dst[5*size] = (float8) (l_mat[0].s5, l_mat[1].s5, l_mat[2].s5, l_mat[3].s5, l_mat[4].s5, l_mat[5].s5, l_mat[6].s5, l_mat[7].s5);
        dst[6*size] = (float8) (l_mat[0].s6, l_mat[1].s6, l_mat[2].s6, l_mat[3].s6, l_mat[4].s6, l_mat[5].s6, l_mat[6].s6, l_mat[7].s6);
        dst[7*size] = (float8) (l_mat[0].s7, l_mat[1].s7, l_mat[2].s7, l_mat[3].s7, l_mat[4].s7, l_mat[5].s7, l_mat[6].s7, l_mat[7].s7);
    }
}


__kernel __attribute__((vec_type_hint(float8))) 
void mult(__global float8 *a, __global float8 *b, __global float *c) {
    int a_row = get_global_id(0);
    int b_row = get_global_id(1);
    int vectors_per_row = SIZE / 8;
    
    a += a_row * vectors_per_row;
    b += b_row * vectors_per_row;
    c += a_row * SIZE + b_row;
    
    float sum = 0.0f;
    for (int i = 0; i < vectors_per_row; i++) {
        sum += dot(a[i].lo, b[i].lo) + dot(a[i].hi, b[i].hi);
    }
    c[0] = sum;
}
