__kernel __attribute__((vec_type_hint(float4)))
void transpose(__global float4 *g_mat) {
    uint size = NUMBLOCKS;
    __global float4 *src, *dst;
    float4 l_mat[4];
    
    int col = get_global_id(0);
    int row = 0;
    while (col >= size) {
        col -= size--;
        row++;
    }
    col += row;
    size += row;
    
    src = g_mat + row * size * 4 + col;
    l_mat[0] = src[0];
    l_mat[1] = src[size];
    l_mat[2] = src[2*size];
    l_mat[3] = src[3*size];
    
    if (row == col) {
        src[0]      = (float4) (l_mat[0].x, l_mat[1].x, l_mat[2].x, l_mat[3].x);
        src[size]   = (float4) (l_mat[0].y, l_mat[1].y, l_mat[2].y, l_mat[3].y);
        src[2*size] = (float4) (l_mat[0].z, l_mat[1].z, l_mat[2].z, l_mat[3].z);
        src[3*size] = (float4) (l_mat[0].w, l_mat[1].w, l_mat[2].w, l_mat[3].w);
    } else {
        dst = g_mat + col * size * 4 + row;
        
        src[0]      = (float4) (dst[0].x, dst[size].x, dst[2*size].x, dst[3*size].x);
        src[size]   = (float4) (dst[0].y, dst[size].y, dst[2*size].y, dst[3*size].y);
        src[2*size] = (float4) (dst[0].z, dst[size].z, dst[2*size].z, dst[3*size].z);
        src[3*size] = (float4) (dst[0].w, dst[size].w, dst[2*size].w, dst[3*size].w);
        
        dst[0]      = (float4) (l_mat[0].x, l_mat[1].x, l_mat[2].x, l_mat[3].x);
        dst[size]   = (float4) (l_mat[0].y, l_mat[1].y, l_mat[2].y, l_mat[3].y);
        dst[2*size] = (float4) (l_mat[0].z, l_mat[1].z, l_mat[2].z, l_mat[3].z);
        dst[3*size] = (float4) (l_mat[0].w, l_mat[1].w, l_mat[2].w, l_mat[3].w);
    }
}

__kernel __attribute__((vec_type_hint(float4))) 
void mult(__global float4 *a, __global float4 *b, __global float *c) {
    int a_row = get_global_id(0);
    int b_row = get_global_id(1);
    int vectors_per_row = SIZE / 4;
    
    a += a_row * vectors_per_row;
    b += b_row * vectors_per_row;
    c += a_row * SIZE + b_row;
    
    float sum = 0.0f;
    for (int i = 0; i < vectors_per_row; i++) {
        sum += dot(a[i], b[i]);
    }
    c[0] = sum;
}
