#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <math.h>

#ifdef __APPLE__
#include <OpenCL/opencl.h>
#else
#include <CL/cl.h>
#endif
#include "../common/cmd_line.c"

float *calculate_on_device(float *a, float *b, size_t n, struct args_t args);

#include "../common/base.c"
#include "../common/opencl_common.c"
#include "../common/files.c"
#ifdef PAPI
#include "../common/performance_papi.c"
#else
#include "../common/performance_no_papi.c"
#endif
#include "../common/run.c"


float *calculate_on_device(float *a, float *b, size_t n, struct args_t args) {
    cl_int err;
    cl_device_id device;
    cl_context context;
    cl_program program;
    cl_kernel kernel;
    cl_command_queue queue;
    cl_mem_flags read, write;

    size_t global_size[2] = {n, n};
    char *build_options = malloc(200);
#ifdef NONE
    sprintf(build_options, "-cl-opt-disable -DSIZE=%ld", n);
#else
    sprintf(build_options, "-DSIZE=%ld", n);
#endif
    
    if (args.device == CL_DEVICE_TYPE_GPU) {
        read = CL_MEM_READ_ONLY | CL_MEM_COPY_HOST_PTR;
        write = CL_MEM_WRITE_ONLY;
    } else {
        read = CL_MEM_READ_ONLY | CL_MEM_USE_HOST_PTR;
        write = CL_MEM_WRITE_ONLY;
    }
    
    device = get_device(args.device);
    context = clCreateContext(NULL, 1, &device, NULL, NULL, &err);
    CHECKERR(err);
    queue = clCreateCommandQueue(context, device, 0, &err);
    CHECKERR(err);
    
#ifdef SSE
    program = build_program(&context, &device, "matmul-sse.cl", build_options, 0);
#endif

#ifdef AVX
    program = build_program(&context, &device, "matmul-avx.cl", build_options, 0);
#endif

#if defined (NONE) || defined(AUTO)
    program = build_program(&context, &device, "matmul-none.cl", build_options, 0);
#endif
    
    kernel = clCreateKernel(program, "matmul", &err);
    CHECKERR(err);
    
    cl_mem a_buffer = clCreateBuffer(context, read, n*n * sizeof(*a), a, &err);
    cl_mem b_buffer = clCreateBuffer(context, read, n*n * sizeof(*b), b, &err);
    cl_mem c_buffer = clCreateBuffer(context, write, n*n * sizeof(float), NULL, &err);
    CHECKERR(err);
    
    err  = clSetKernelArg(kernel, 0, sizeof(cl_mem), &a_buffer);
    err |= clSetKernelArg(kernel, 1, sizeof(cl_mem), &b_buffer);
    err |= clSetKernelArg(kernel, 2, sizeof(cl_mem), &c_buffer);
    CHECKERR(err);
    
    err = clEnqueueNDRangeKernel(queue, kernel, 2, NULL, global_size, NULL, 0, NULL, NULL);
    CHECKERR(err);
    
    float *result = calloc(n*n, sizeof(*result));
    err = clEnqueueReadBuffer(queue, c_buffer, CL_FALSE, 0, n*n * sizeof(*a), result, 0, NULL, NULL);
    CHECKERR(err);
    
    clFinish(queue);
    
    free(build_options);
    clReleaseMemObject(a_buffer);
    clReleaseMemObject(b_buffer);
    clReleaseMemObject(c_buffer);
    clReleaseKernel(kernel);
    clReleaseProgram(program);
    clReleaseCommandQueue(queue);
    clReleaseContext(context);
    
    return result;
}

int main (int argc, const char **argv) {
    run(argc, argv);
    return 0;
}
